@@ -1,17 +1,31 @@
-import requests
-import io
-import zipfile
-
 from django.conf import settings
 from django.contrib import admin
 from django.http import HttpResponse
+from django.contrib.auth.admin import UserAdmin
+from django.contrib.auth import get_user_model
 from django.template.response import TemplateResponse
 from django.utils.safestring import mark_safe
 from reversion.admin import VersionAdmin
 from django.urls import path
+from django.shortcuts import render, redirect, reverse
 
+from .generator.forms import QrGenerateForm
+from .generator.zip import generate_qr_zip
 from .models import ApiHit, Department, LinkUrl, QRCode
 
+User = get_user_model()
+
+admin.site.unregister(User)
+
+
+class DepartmentInline(admin.TabularInline):
+    model = Department.users.through
+
+
+@admin.register(User)
+class MyUserAdmin(UserAdmin):
+    inlines = [DepartmentInline]
+
 
 @admin.register(ApiHit)
 class ApiHitAdmin(admin.ModelAdmin):
@@ -34,17 +48,17 @@ class QRCodeAdmin(VersionAdmin):
     list_display = ('title', 'department',
                     'get_code_url', 'get_code_image_url')
     list_filter = (('department', admin.RelatedOnlyFieldListFilter),)
-    search_fields = ('title', 'department__name')
+    search_fields = ('title', 'department__name', 'basic_info')
     inlines = [LinkUrlInline]
     change_list_template = 'qrtoolkit_core/qrcode/change_list.html'
-    actions = ['download_codes', ]
+    actions = ['download_codes_action', ]
     readonly_fields = ('uuid',)
 
     def get_queryset(self, request):
         qs = super(QRCodeAdmin, self).get_queryset(request)
         if request.user.is_superuser:
             return qs
-        return qs.filter(department=request.user.department)
+        return qs.filter(department__in=request.user.departments.all())
 
     def get_model_perms(self, request):
         return super(QRCodeAdmin, self).get_model_perms(request)
@@ -60,37 +74,22 @@ class QRCodeAdmin(VersionAdmin):
     get_code_image_url.short_description = 'Code image'
     get_code_url.short_description = 'Code url'
 
-    def download_codes(self, request, queryset):
-        zip_filename = 'archive.zip'
-        s = io.BytesIO()
+    def download_codes_action(self, request, queryset):
+        pks = [str(x.pk) for x in queryset.all()]
+        url = '%s?codes=%s' % (reverse('admin:admin_qr_download'), ','.join(pks))
+        return redirect(url)
 
-        zf = zipfile.ZipFile(s, mode='w', compression=zipfile.ZIP_DEFLATED)
-
-        downloaded_files = []
-        for code in queryset.all():
-            url = settings.REDIRECT_SERVICE_URL + f'/code/{code.short_uuid}/dl/'
-            res = requests.get(url)
-            zf.writestr(f'{code.title}-{code.uuid}.svg', res.content)
-            downloaded_files.append(code.title)
-
-        zf.close()
-
-        resp = HttpResponse(s.getvalue(), content_type="application/x-zip-compressed")
-        resp['Content-Disposition'] = f'attachment; filename={zip_filename}'
-
-        message = f'Downloaded zip with files: {", ".join(downloaded_files)}'
-        self.message_user(request, message=message)
-        return resp
+    download_codes_action.short_description = 'Download codes'
 
     def formfield_for_foreignkey(self, db_field, request, **kwargs):
         if db_field.name == 'department' and not request.user.is_superuser:
-            kwargs['queryset'] = Department.objects.filter(name__exact=request.user.department.name)
+            kwargs['queryset'] = request.user.departments
         return super(QRCodeAdmin, self).formfield_for_foreignkey(db_field, request, **kwargs)
 
     def get_urls(self):
         urls = super(QRCodeAdmin, self).get_urls()
         custom_urls = [
-            path('scan/', self.admin_site.admin_view(self.scan_view))
+            path('download/', self.admin_site.admin_view(self.download_codes), name='admin_qr_download')
         ]
         return custom_urls + urls
 
@@ -105,6 +104,36 @@ class QRCodeAdmin(VersionAdmin):
 
         return TemplateResponse(request, 'qrtoolkit_core/qrcode/scanner.html', context)
 
+    def download_codes(self, request):
+        pks = [int(x) for x in request.GET['codes'].split(',')]
+        codes = QRCode.objects.filter(pk__in=pks, department__in=request.user.departments.all()).all()
+        if request.method == 'POST':
+            form = QrGenerateForm(data=request.POST)
+            if form.is_valid():
+                zf = generate_qr_zip(codes, form)
+                resp = HttpResponse(zf, content_type="application/x-zip-compressed")
+                resp['Content-Disposition'] = f'attachment; filename=generated_codes.zip'
+                return resp
+            else:
+                context = dict(
+                    self.admin_site.each_context(request),
+                    is_nav_sidebar_enabled=False,
+                    title="Download Qr codes",
+                    codes=codes,
+                    form=form
+                )
+                return render(request, 'qrtoolkit_core/qrcode/download.html', context=context)
+        else:
+            context = dict(
+                self.admin_site.each_context(request),
+                is_nav_sidebar_enabled=False,
+                title="Download Qr codes",
+                codes=codes,
+                form=QrGenerateForm()
+            )
+
+            return render(request, 'qrtoolkit_core/qrcode/download.html', context=context)
+
 
 @admin.register(Department)
 class DepartmentAdmin(admin.ModelAdmin):
